package db

import (
	"encoding/json"
	"testing"

	"github.com/aquasecurity/trivy-db/pkg/types"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestConfig_GetVulnerabilityDetail(t *testing.T) {
	olddbcForEachFunc := dbcForEachFunc
	defer func() {
		dbcForEachFunc = olddbcForEachFunc
	}()

	dbcForEachFunc = func(rootBucket, nestedBucket string) (value map[string][]byte, err error) {

		vRedhat, _ := json.Marshal(types.VulnerabilityDetail{
			ID:          "CVE-2020-1234",
			CvssScore:   4.3,
			CvssScoreV3: 5.6,
			Severity:    types.SeverityHigh,
			SeverityV3:  types.SeverityCritical,
			Title:       "test vulnerability",
			Description: "a test vulnerability where vendor rates it lower than NVD",
		})

		vUbuntu, _ := json.Marshal(types.VulnerabilityDetail{
			ID:          "CVE-2020-1234",
			CvssScore:   1.2,
			CvssScoreV3: 3.4,
			Severity:    types.SeverityLow,
			SeverityV3:  types.SeverityMedium,
			Title:       "test vulnerability",
			Description: "a test vulnerability where vendor rates it lower than NVD",
		})

		return map[string][]byte{
			"redhat": vRedhat,
			"ubuntu": vUbuntu,
		}, nil
	}

	vd, err := Config{}.GetVulnerabilityDetail("CVE-2020-1234")
	require.NoError(t, err)
	assert.Equal(t, map[string]types.VulnerabilityDetail{
		"redhat": {
			ID:          "CVE-2020-1234",
			CvssScore:   4.3,
			CvssScoreV3: 5.6,
			Severity:    types.SeverityHigh,
			SeverityV3:  types.SeverityCritical,
			Title:       "test vulnerability",
			Description: "a test vulnerability where vendor rates it lower than NVD",
		},
		"ubuntu": {
			ID:          "CVE-2020-1234",
			CvssScore:   1.2,
			CvssScoreV3: 3.4,
			Severity:    types.SeverityLow,
			SeverityV3:  types.SeverityMedium,
			Title:       "test vulnerability",
			Description: "a test vulnerability where vendor rates it lower than NVD",
		},
	}, vd)
}
